@@ -5,6 +5,9 @@ import string
 
 import pypandoc
 
+html_convert = lambda x: pypandoc.convert_text(x, "html5", "md", ["--mathjax"])
+tex_convert = lambda x: pypandoc.convert_text(x, "latex", "md")
+
 
 class LineBuffer:
     def __init__(self, text):
@@ -21,11 +24,18 @@ class LineBuffer:
         return self.i == len(self.lines)
 
 
-def directive_type(line):
-    if not any(line.startswith(f"# {x} ") for x in ["BEGIN", "END", "INPUT", "CONFIG", "DEFINE"]):
+def parse_directive(line):
+    if not any(
+            line.startswith(f"# {x} ")
+            for x in ["BEGIN", "END", "INPUT", "CONFIG", "DEFINE"]
+    ):
         return None, None, None
     tokens = line.split(" ", 3)
-    return tokens[1], tokens[2] if len(tokens) > 2 else "", tokens[3] if len(tokens) > 3 else ""
+    return (
+        tokens[1],
+        tokens[2] if len(tokens) > 2 else "",
+        tokens[3] if len(tokens) > 3 else "",
+    )
 
 
 def process_title(line):
@@ -44,16 +54,26 @@ def process_title(line):
 
 
 def rand_id():
-    return ''.join(random.choices(string.ascii_uppercase, k=32))
+    return "".join(random.choices(string.ascii_uppercase, k=32))
+
+
+class ToParse():
+    def __init__(self, text, type):
+        self.text = text
+        self.type = type
+
+        self.html = None
+        self.tex = None
 
 
 def parse(text):
     return {
-        "html": pypandoc.convert_text(text, "html5", "md", ["--mathjax"]),
-        "tex": pypandoc.convert_text(text, "latex", "md"),
         "text": text,
+        "html": html_convert(text),  # ToParse(text, "html"),
+        "tex": tex_convert(text)  # ToParse(text, "tex"),
     }
 
+
 def parse_define(directive, rest, substitutions, substitutions_match):
     if directive == "MATCH":
         regex = r"\[(.*)\]\s+\[(.*)\]"
@@ -64,59 +84,101 @@ def parse_define(directive, rest, substitutions, substitutions_match):
         directives_list = directives.split(" ")
         replacements_list = replacements.split(" ")
         if len(replacements_list) < len(directives_list):
-            raise SyntaxError("DEFINE MATCH must have at least as many replacements as it has directives")
+            raise SyntaxError(
+                "DEFINE MATCH must have at least as many replacements as it has directives"
+            )
         substitutions_match.append(
-            {
-                "directives": directives_list, 
-                "replacements": replacements_list,
-            }
+            {"directives": directives_list, "replacements": replacements_list}
         )
     else:
         substitutions[directive] = rest.split(" ")
 
+
 def parse_input_lines(lines):
     if not lines:
         raise SyntaxError("No INPUT directives found in QUESTION")
-    _, directive, rest = directive_type(lines[0])
+    _, directive, rest = parse_directive(lines[0])
+    correct_options = []
     if directive == "OPTION" or directive == "SELECT":
         options = []
         for line in lines:
-            _, other_directive, rest = directive_type(line)
+            _, other_directive, rest = parse_directive(line)
             if other_directive != directive:
                 raise SyntaxError("Multiple INPUT types found in a single QUESTION")
             fixed = "FIXED "
+            is_fixed = False
             if rest.startswith(fixed):
-                options.append(parse(rest[len(fixed):]))
-                options[-1]["fixed"] = True
-            else:
-                options.append(parse(rest))
-        return "multiple_choice" if directive == "OPTION" else "select_all", options
-    elif directive in ("SHORT_ANSWER", "SHORT_CODE_ANSWER", "LONG_ANSWER", "LONG_CODE_ANSWER"):
+                is_fixed = True
+                rest = rest[len(fixed):]
+
+            correct = "CORRECT "
+            is_correct = False
+            if rest.startswith(correct):
+                is_correct = True
+                rest = rest[len(correct):]
+
+            if is_correct:
+                correct_options.append(rest)
+
+            options.append(parse(rest))
+            options[-1]["fixed"] = is_fixed
+        return (
+            "multiple_choice" if directive == "OPTION" else "select_all",
+            options,
+            correct_options,
+        )
+    elif directive in (
+            "SHORT_ANSWER",
+            "SHORT_CODE_ANSWER",
+            "LONG_ANSWER",
+            "LONG_CODE_ANSWER",
+    ):
         if len(lines) > 1:
-            raise SyntaxError("Multiple INPUT directives found for a {}".format(directive))
+            raise SyntaxError(
+                "Multiple INPUT directives found for a {}".format(directive)
+            )
         if directive == "SHORT_ANSWER":
-            return "short_answer", None
+            return "short_answer", None, None
         elif directive == "SHORT_CODE_ANSWER":
-            return "short_code_answer", None
+            return "short_code_answer", None, None
         try:
             num_lines = int(rest or "10")
         except TypeError:
             raise SyntaxError("Expected integer as option for {}".format(directive))
         if directive == "LONG_ANSWER":
-            return "long_answer", num_lines
+            return "long_answer", num_lines, None
         elif directive == "LONG_CODE_ANSWER":
-            return "long_code_answer", num_lines
+            return "long_code_answer", num_lines, None
     raise SyntaxError("Unrecognized directive: {}".format(directive))
 
 
+def consume_rest_of_solution(buff, end):
+    out = []
+    while True:
+        line = buff.pop()
+        mode, directive, rest = parse_directive(line)
+        if mode is None:
+            out.append(line)
+        elif mode == "END":
+            if directive == end:
+                return parse("\n".join(out))
+            else:
+                raise SyntaxError("Unexpected END in SOLUTION")
+        else:
+            raise SyntaxError("Unexpected directive in SOLUTION")
+
+
 def consume_rest_of_question(buff):
     contents = []
     input_lines = []
     substitutions = {}
     substitutions_match = []
+    solution = None
+    solution_note = None
+    config = {}
     while True:
         line = buff.pop()
-        mode, directive, rest = directive_type(line)
+        mode, directive, rest = parse_directive(line)
         if mode is None:
             if input_lines and line.strip():
                 raise SyntaxError("Unexpected content in QUESTION after INPUT")
@@ -124,13 +186,32 @@ def consume_rest_of_question(buff):
                 contents.append(line)
         elif mode == "INPUT":
             input_lines.append(line)
+        elif mode == "BEGIN":
+            if directive == "SOLUTION":
+                solution = consume_rest_of_solution(buff, directive)
+            elif directive == "NOTE":
+                solution_note = consume_rest_of_solution(buff, directive)
+            else:
+                raise SyntaxError("Unexpected BEGIN in QUESTION")
         elif mode == "END":
             if directive == "QUESTION":
-                question_type, options = parse_input_lines(input_lines)
+                question_type, options, option_solutions = parse_input_lines(
+                    input_lines
+                )
+
+                if option_solutions and solution:
+                    raise SyntaxError("Received multiple solutions.")
+
                 return {
                     "id": rand_id(),
                     "type": question_type,
+                    "solution": {
+                        "solution": solution,
+                        "options": option_solutions,
+                        "note": solution_note,
+                    },
                     **parse("\n".join(contents)),
+                    "config": config,
                     "options": options,
                     "substitutions": substitutions,
                     "substitutions_match": substitutions_match,
@@ -139,6 +220,8 @@ def consume_rest_of_question(buff):
                 raise SyntaxError("Unexpected END in QUESTION")
         elif mode == "DEFINE":
             parse_define(directive, rest, substitutions, substitutions_match)
+        elif mode == "CONFIG":
+            config[directive] = rest
         else:
             raise SyntaxError("Unexpected directive in QUESTION")
 
@@ -153,7 +236,7 @@ def consume_rest_of_group(buff, end):
     scramble = False
     while True:
         line = buff.pop()
-        mode, directive, rest = directive_type(line)
+        mode, directive, rest = parse_directive(line)
         if mode is None:
             if started_elements and line.strip():
                 raise SyntaxError("Unexpected text in GROUP after QUESTIONs started")
@@ -163,7 +246,9 @@ def consume_rest_of_group(buff, end):
             started_elements = True
             title, is_fixed, points = process_title(rest)
             if title:
-                raise SyntaxError("Unexpected arguments passed in BEGIN QUESTION directive")
+                raise SyntaxError(
+                    "Unexpected arguments passed in BEGIN QUESTION directive"
+                )
             question = consume_rest_of_question(buff)
             question["points"] = points
             question["fixed"] = is_fixed
@@ -204,7 +289,7 @@ def consume_rest_of_group(buff, end):
             raise SyntaxError("Unexpected directive in GROUP")
 
 
-def convert(text):
+def _convert(text):
     buff = LineBuffer(text)
     groups = []
     public = None
@@ -217,12 +302,18 @@ def convert(text):
             line = buff.pop()
             if not line.strip():
                 continue
-            mode, directive, rest = directive_type(line)
+            mode, directive, rest = parse_directive(line)
             if mode == "CONFIG":
-                if directive in ["SCRAMBLE_GROUPS", "SCRAMBLE_QUESTIONS", "SCRAMBLE_OPTIONS"]:
+                if directive in [
+                    "SCRAMBLE_GROUPS",
+                    "SCRAMBLE_QUESTIONS",
+                    "SCRAMBLE_OPTIONS",
+                ]:
                     config[directive.lower()] = [int(x) for x in rest.split(" ") if x]
                 else:
-                    raise SyntaxError("Unexpected CONFIG directive {}".format(directive))
+                    raise SyntaxError(
+                        "Unexpected CONFIG directive {}".format(directive)
+                    )
             elif mode == "BEGIN" and directive in ["GROUP", "PUBLIC"]:
                 title, is_fixed, points = process_title(rest)
                 group = consume_rest_of_group(buff, directive)
@@ -253,5 +344,46 @@ def convert(text):
     }
 
 
+def pandoc(target):
+    to_parse = []
+
+    def explore(pos):
+        if isinstance(pos, ToParse):
+            to_parse.append(pos)
+        elif isinstance(pos, dict):
+            for child in pos.values():
+                explore(child)
+        elif isinstance(pos, list):
+            for child in pos:
+                explore(child)
+
+    explore(target)
+
+    DELIMITER = """\n\nDELIMITER\n\n"""
+
+    transpile_target = lambda t: DELIMITER.join(x.text for x in to_parse if x.type == t)
+
+    html = html_convert(transpile_target("html")).split(html_convert(DELIMITER))
+    tex = tex_convert(transpile_target("tex")).split(tex_convert(DELIMITER))
+
+    assert not html or not tex or (len(to_parse) == len(html) + len(tex), (len(to_parse), len(html), len(tex)))
+
+    for x, h in zip(filter(lambda x: x.type == "html", to_parse), html):
+        x.html = h
+
+    for x, t in zip(filter(lambda x: x.type == "tex", to_parse), tex):
+        x.tex = t
+
+    def pandoc_dump(obj):
+        assert isinstance(obj, ToParse)
+        return obj.__dict__[obj.type]
+
+    return json.dumps(target, default=pandoc_dump)
+
+
+def convert(text):
+    return json.loads(convert_str(text))
+
+
 def convert_str(text):
-    return json.dumps(convert(text))
+    return pandoc(_convert(text))
