@@ -63,11 +63,11 @@ def scramble(email, exam, *, keep_data=False):
 
         if keep_data and "solution" in question:
             solution = question["solution"]
-            if solution.get("solution"):
+            if solution.get("solution") is not None:
                 substitute(
                     solution["solution"], [question_substitutions, *substitutions], ["html", "tex", "text"], store=False
                 )
-            else:
+            elif solution["options"] is not None:
                 substitute(
                     solution["options"],
                     [question_substitutions, *substitutions],
